@@ -28,13 +28,13 @@ test_that("Test initials, A and B of CES on N2568$x", {
 x <- cbind(c(rep(0,25),1,rep(0,43)),c(rep(0,10),1,rep(0,58)));
 y <- ts(c(Mcomp::M3$N1457$x,Mcomp::M3$N1457$xx),frequency=12);
 testModel <- ces(y, h=18, holdout=TRUE, xreg=x, updateX=TRUE, silent=TRUE, cfType="aMSTFE", intervals="sp")
-test_that("Check exogenous variables for CES on N1457", {
+test_that("Check exogenous variables for CESX on N1457", {
     expect_equal(suppressWarnings(ces(y, h=18, holdout=TRUE, xreg=x, silent=TRUE)$model), testModel$model);
-    expect_equal(suppressWarnings(forecast(testModel, h=18, holdout=FALSE)$model), testModel$model);
+    expect_equal(suppressWarnings(forecast(testModel, h=18, holdout=FALSE)$method), testModel$model);
 })
 
 # Test selection of exogenous with CES
 testModel <- ces(y, h=18, holdout=TRUE, xreg=x, silent=TRUE, xregDo="select")
-test_that("Select exogenous variables for CES on N1457", {
+test_that("Select exogenous variables for CESX on N1457 with selection", {
     expect_equal(suppressWarnings(sum(testModel$xreg)),1);
 })
